@@ -1,5 +1,18 @@
 #include "links.h"
 
+#ifdef DONT_USE_AF_UNIX
+
+int bind_to_af_unix()
+{
+	return -1;
+}
+
+void af_unix_close()
+{
+}
+
+#else
+
 #ifdef USE_AF_UNIX
 #include <sys/un.h>
 #endif
@@ -17,6 +30,7 @@ int get_address()
 {
 	struct sockaddr_un *su;
 	unsigned char *path;
+	if (!links_home) return -1;
 	path = stracpy(links_home);
 	if (!(su = mem_alloc(sizeof(struct sockaddr_un) + strlen(path) + 1))) {
 		mem_free(path);
@@ -135,3 +149,4 @@ void af_unix_close()
 	if (s_unix_acc) mem_free(s_unix_acc), s_unix_acc = NULL;
 }
 
+#endif
