@@ -46,9 +46,6 @@ void freeml(struct memory_list *ml)
 
 unsigned char m_bar = 0;
 
-#define DIALOG_LB	(DIALOG_LEFT_BORDER + DIALOG_LEFT_INNER_BORDER + 1)
-#define DIALOG_TB	(DIALOG_TOP_BORDER + DIALOG_TOP_INNER_BORDER + 1)
-
 void menu_func(struct window *, struct event *, int);
 void mainmenu_func(struct window *, struct event *, int);
 void dialog_func(struct window *, struct event *, int);
@@ -153,7 +150,7 @@ void display_menu(struct terminal *term, struct menu *menu)
 		int x;
 		int h = 0;
 		unsigned char c;
-		char *tmptext = _(menu->items[p].text, term);
+		unsigned char *tmptext = _(menu->items[p].text, term);
 		int co = p == menu->selected ? h = 1, COLOR_MENU_SELECTED : COLOR_MENU;
 		if (h) {
 			set_cursor(term, menu->x+1, s, term->x - 1, term->y - 1);
@@ -219,53 +216,58 @@ void menu_func(struct window *win, struct event *ev, int fwd)
 			}
 			break;
 		case EV_KBD:
-			if (ev->x == KBD_LEFT || ev->x == KBD_RIGHT) {
-				if ((void *)win->next != &win->term->windows && win->next->handler == mainmenu_func) goto mm;
-				/*for (w1 = win; (void *)w1 != &win->term->windows; w1 = w1->next) {
-					if (w1->handler == mainmenu_func) goto mm;
-					if (w1->handler != menu_func) break;
-				}*/
-				if (ev->x == KBD_RIGHT) goto enter;
-				delete_window(win);
-				break;
-			}
-			if ((ev->x >= KBD_F1 && ev->x <= KBD_F12) || ev->y == KBD_ALT) {
-				mm:
-				delete_window_ev(win, ev);
-				break;
-			}
-			if (ev->x == KBD_ESC) {
-				delete_window_ev(win, (void *)win->next != &win->term->windows && win->next->handler == mainmenu_func ? ev : NULL);
-				break;
-			}
-			if (ev->x == KBD_UP) scroll_menu(menu, -1);
-			if (ev->x == KBD_DOWN) scroll_menu(menu, 1);
-			if (ev->x == KBD_HOME) menu->selected = -1, scroll_menu(menu, 1);
-			if (ev->x == KBD_END) menu->selected = menu->ni, scroll_menu(menu, -1);
-			if (ev->x == KBD_PAGE_UP) {
-				if ((menu->selected -= menu->yw - 3) < -1) menu->selected = -1;
-				if ((menu->view -= menu->yw - 2) < 0) menu->view = 0;
-				scroll_menu(menu, -1);
-			}
-			if (ev->x == KBD_PAGE_DOWN) {
-				if ((menu->selected += menu->yw - 3) > menu->ni) menu->selected = menu->ni;
-				if ((menu->view += menu->yw - 2) >= menu->ni - menu->yw + 2) menu->view = menu->ni - menu->yw + 2;
-				scroll_menu(menu, 1);
-			}
-			if (ev->x > ' ' && ev->x < 256) {
-				int i;
-				for (i = 0; i < menu->ni; i++)
-					if (strchr(_(menu->items[i].hotkey, win->term), upcase(ev->x))) {
-						menu->selected = i;
-						scroll_menu(menu, 0);
-						s = 1;
+			switch (kbd_action(KM_MENU, ev)) {
+				case ACT_LEFT:
+				case ACT_RIGHT:
+					if ((void *)win->next != &win->term->windows && win->next->handler == mainmenu_func) goto mm;
+					/*for (w1 = win; (void *)w1 != &win->term->windows; w1 = w1->next) {
+						if (w1->handler == mainmenu_func) goto mm;
+						if (w1->handler != menu_func) break;
+					}*/
+					if (kbd_action(KM_MENU, ev) == ACT_RIGHT) goto enter;
+					delete_window(win);
+					goto break2;
+				case ACT_UP: scroll_menu(menu, -1); break;
+				case ACT_DOWN: scroll_menu(menu, 1); break;
+				case ACT_HOME: menu->selected = -1, scroll_menu(menu, 1); break;
+				case ACT_END: menu->selected = menu->ni, scroll_menu(menu, -1); break;
+				case ACT_PAGE_UP:
+					if ((menu->selected -= menu->yw - 3) < -1) menu->selected = -1;
+					if ((menu->view -= menu->yw - 2) < 0) menu->view = 0;
+					scroll_menu(menu, -1);
+					break;
+				case ACT_PAGE_DOWN:
+					if ((menu->selected += menu->yw - 3) > menu->ni) menu->selected = menu->ni;
+					if ((menu->view += menu->yw - 2) >= menu->ni - menu->yw + 2) menu->view = menu->ni - menu->yw + 2;
+					scroll_menu(menu, 1);
+					break;
+				default:
+					if ((ev->x >= KBD_F1 && ev->x <= KBD_F12) || ev->y == KBD_ALT) {
+						mm:
+						delete_window_ev(win, ev);
+						goto break2;
 					}
+					if (ev->x == KBD_ESC) {
+						delete_window_ev(win, (void *)win->next != &win->term->windows && win->next->handler == mainmenu_func ? ev : NULL);
+						goto break2;
+					}
+					if (ev->x > ' ' && ev->x < 256) {
+						int i;
+						for (i = 0; i < menu->ni; i++)
+							if (strchr(_(menu->items[i].hotkey, win->term), upcase(ev->x))) {
+								menu->selected = i;
+								scroll_menu(menu, 0);
+								s = 1;
+							}
+					}
+					break;
 			}
 			display_menu(win->term, menu);
 			if (s || ev->x == KBD_ENTER || ev->x == ' ') {
 				enter:
 				select_menu(win->term, menu);
 			}
+			break2:
 			break;
 		case EV_ABORT:
 			if (menu->items->free_i) {
@@ -305,7 +307,7 @@ void display_mainmenu(struct terminal *term, struct mainmenu *menu)
 		int s = 0;
 		int j;
 		unsigned char c;
-		char *tmptext = _(menu->items[i].text, term);
+		unsigned char *tmptext = _(menu->items[i].text, term);
 		int co = i == menu->selected ? s = 1, COLOR_MAINMENU_SELECTED : COLOR_MAINMENU;
 		if (s) {
 			fill_area(term, p, 0, 2, 1, co);
@@ -352,7 +354,7 @@ void mainmenu_func(struct window *win, struct event *ev, int fwd)
 				int p = 2;
 				for (i = 0; i < menu->ni; i++) {
 					int o = p;
-					char *tmptext = _(menu->items[i].text, win->term);
+					unsigned char *tmptext = _(menu->items[i].text, win->term);
 					p += strlen(tmptext) + 4;
 					if (ev->x >= o && ev->x < p) {
 						menu->selected = i;
@@ -453,12 +455,16 @@ void display_dlg_item(struct dialog_data *dlg, struct dialog_item_data *di, int
 				set_window_ptr(dlg->win, di->x, di->y);
 			}
 			break;
+		case D_FIELD_PASS:
 		case D_FIELD:
 			if (di->vpos + di->l <= di->cpos) di->vpos = di->cpos - di->l + 1;
 			if (di->vpos > di->cpos) di->vpos = di->cpos;
 			if (di->vpos < 0) di->vpos = 0;
 			fill_area(term, di->x, di->y, di->l, 1, COLOR_DIALOG_FIELD);
-			print_text(term, di->x, di->y, strlen(di->cdata + di->vpos) <= di->l ? strlen(di->cdata + di->vpos) : di->l, di->cdata + di->vpos, COLOR_DIALOG_FIELD_TEXT);
+			if (di->item->type == D_FIELD) {
+				print_text(term, di->x, di->y, strlen(di->cdata + di->vpos) <= di->l ? strlen(di->cdata + di->vpos) : di->l, di->cdata + di->vpos, COLOR_DIALOG_FIELD_TEXT);
+			}
+			else fill_area(term, di->x, di->y, strlen(di->cdata + di->vpos) <= di->l ? strlen(di->cdata + di->vpos) : di->l, 1, COLOR_DIALOG_FIELD_TEXT | '*');
 			if (sel) {
 				set_cursor(term, di->x + di->cpos - di->vpos, di->y, di->x + di->cpos - di->vpos, di->y);
 				set_window_ptr(dlg->win, di->x, di->y);
@@ -525,6 +531,7 @@ int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struct event
 			display_dlg_item(dlg, di, 1);
 			if ((ev->b & BM_ACT) == B_UP) dlg_select_item(dlg, di);
 			return 1;
+		case D_FIELD_PASS:
 		case D_FIELD:
 			if (ev->y != di->y || ev->x < di->x || ev->x >= di->x + di->l) return 0;
 			if ((di->cpos = di->vpos + ev->x - di->x) > strlen(di->cdata)) di->cpos = strlen(di->cdata);
@@ -569,6 +576,56 @@ void redraw_dialog(struct dialog_data *dlg)
 	redraw_from_window(dlg->win);
 }
 
+void tab_compl(struct terminal *term, unsigned char *item, struct window *win)
+{
+	struct event ev = {EV_REDRAW, 0, 0, 0};
+	struct dialog_item_data *di = &((struct dialog_data*)win->data)->items[((struct dialog_data*)win->data)->selected];
+	int l = strlen(item);
+	if (l >= di->item->dlen) l = di->item->dlen - 1;
+	memcpy(di->cdata, item, l);
+	di->cdata[l] = 0;
+	di->cpos = l;
+	di->vpos = 0;
+	ev.x = term->x;
+	ev.y = term->y;
+	dialog_func(win, &ev, 0);
+}
+
+void do_tab_compl(struct terminal *term, struct list_head *history, struct window *win)
+{
+	unsigned char *cdata = ((struct dialog_data*)win->data)->items[((struct dialog_data*)win->data)->selected].cdata;
+	int l = strlen(cdata), n = 0;
+	struct history_item *hi;
+	struct menu_item *items = DUMMY, *i;
+	foreach(hi, *history) if (!strncmp(cdata, hi->d, l)) {
+		if (!(n & (ALLOC_GR - 1))) {
+			if (!(i = mem_realloc(items, (n + ALLOC_GR + 1) * sizeof(struct menu_item)))) {
+				mem_free(items);
+				return;
+			}
+			items = i;
+		}
+		items[n].text = hi->d;
+		items[n].rtext = "";
+		items[n].hotkey = "";
+		items[n].func = (void(*)(struct terminal *, void *, void *))tab_compl;
+		items[n].rtext = "";
+		items[n].data = hi->d;
+		items[n].in_m = 0;
+		items[n].free_i = 1;
+		n++;
+	}
+	if (n == 1) {
+		tab_compl(term, items->data, win);
+		mem_free(items);
+		return;
+	}
+	if (n) {
+		memset(&items[n], 0, sizeof(struct menu_item));
+		do_menu(term, items, win);
+	}
+}
+
 void dialog_func(struct window *win, struct event *ev, int fwd)
 {
 	int i;
@@ -607,7 +664,7 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
 				}
 				init_list(di->history);
 				di->cur_hist = (struct history_item *)&di->history;
-				if (di->item->type == D_FIELD) {
+				if (di->item->type == D_FIELD || di->item->type == D_FIELD_PASS) {
 					if (di->item->history) {
 						struct history_item *j;
 						/*int l = di->item->dlen;*/
@@ -632,78 +689,81 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
 			break;
 		case EV_KBD:
 			di = &dlg->items[dlg->selected];
-			if (di->item->type == D_FIELD) {
-				if (ev->x == KBD_UP && (void *)di->cur_hist->prev != &di->history) {
-					di->cur_hist = di->cur_hist->prev;
-					dlg_set_history(di);
-					goto dsp_f;
-				}
-				if (ev->x == KBD_DOWN && (void *)di->cur_hist != &di->history) {
-					di->cur_hist = di->cur_hist->next;
-					dlg_set_history(di);
-					goto dsp_f;
-				}
-				if (ev->x == KBD_RIGHT) {
-					if (di->cpos < strlen(di->cdata)) di->cpos++;
-					goto dsp_f;
-				}
-				if (ev->x == KBD_LEFT) {
-					if (di->cpos > 0) di->cpos--;
-					goto dsp_f;
-				}
-				if (ev->x == KBD_HOME) {
-					di->cpos = 0;
-					goto dsp_f;
-				}
-				if (ev->x == KBD_END) {
-					di->cpos = strlen(di->cdata);
-					goto dsp_f;
-				}
-				if (ev->x >= ' ' && ev->x < 0x100 && !ev->y) {
-					if (strlen(di->cdata) < di->item->dlen - 1) {
-						memmove(di->cdata + di->cpos + 1, di->cdata + di->cpos, strlen(di->cdata) - di->cpos + 1);
-						di->cdata[di->cpos++] = ev->x;
+			if (di->item->type == D_FIELD || di->item->type == D_FIELD_PASS) {
+				switch (kbd_action(KM_EDIT, ev)) {
+					case ACT_UP:
+						if ((void *)di->cur_hist->prev != &di->history) {
+							di->cur_hist = di->cur_hist->prev;
+							dlg_set_history(di);
+							goto dsp_f;
+						}
+						break;
+					case ACT_DOWN:
+						if ((void *)di->cur_hist != &di->history) {
+							di->cur_hist = di->cur_hist->next;
+							dlg_set_history(di);
+							goto dsp_f;
+						}
+						break;
+					case ACT_RIGHT:
+						if (di->cpos < strlen(di->cdata)) di->cpos++;
+						goto dsp_f;
+					case ACT_LEFT:
+						if (di->cpos > 0) di->cpos--;
+						goto dsp_f;
+					case ACT_HOME:
+						di->cpos = 0;
+						goto dsp_f;
+					case ACT_END:
+						di->cpos = strlen(di->cdata);
+						goto dsp_f;
+					case ACT_BACKSPACE:
+						if (di->cpos) {
+							memmove(di->cdata + di->cpos - 1, di->cdata + di->cpos, strlen(di->cdata) - di->cpos + 1);
+							di->cpos--;
+						}
+						goto dsp_f;
+					case ACT_DELETE:
+						if (di->cpos < strlen(di->cdata))
+							memmove(di->cdata + di->cpos, di->cdata + di->cpos + 1, strlen(di->cdata) - di->cpos + 1);
+						goto dsp_f;
+					case ACT_KILL_TO_BOL:
+						memmove(di->cdata, di->cdata + di->cpos, strlen(di->cdata + di->cpos) + 1);
+						di->cpos = 0;
+						goto dsp_f;
+					case ACT_KILL_TO_EOL:
+						di->cdata[di->cpos] = 0;
+						goto dsp_f;
+				    	case ACT_COPY_CLIPBOARD:
+ 						/* Copy to clipboard */
+						set_clipboard_text(di->cdata);
+						break;	/* We don't need to redraw */
+					case ACT_CUT_CLIPBOARD:
+ 						/* Cut to clipboard */						
+						set_clipboard_text(di->cdata);
+						di->cdata[0] = 0;
+						di->cpos = 0;
+						goto dsp_f;
+					case ACT_PASTE_CLIPBOARD: {
+						/* Paste from clipboard */
+						unsigned char * clipboard = get_clipboard_text();
+						strncpy(di->cdata , clipboard, di->item->dlen);
+						di->cdata[di->item->dlen - 1] = 0;
+						di->cpos = strlen(di->cdata);
+						mem_free(clipboard);
+						goto dsp_f;
 					}
-					goto dsp_f;
-				}
-				if (ev->x == KBD_BS) {
-					if (di->cpos) {
-						memmove(di->cdata + di->cpos - 1, di->cdata + di->cpos, strlen(di->cdata) - di->cpos + 1);
-						di->cpos--;
-					}
-					goto dsp_f;
-				}
-				if (ev->x == KBD_DEL) {
-					if (di->cpos < strlen(di->cdata))
-						memmove(di->cdata + di->cpos, di->cdata + di->cpos + 1, strlen(di->cdata) - di->cpos + 1);
-					goto dsp_f;
-				}
-				if (upcase(ev->x) == 'U' && ev->y == KBD_CTRL) {
-					memmove(di->cdata, di->cdata + di->cpos, strlen(di->cdata + di->cpos) + 1);
-					di->cpos = 0;
-					goto dsp_f;
-				}
-				/* Copy to clipboard */
-				if (ev->x == KBD_INS && ev->y == KBD_CTRL || upcase(ev->x) == 'C' && ev->y == KBD_CTRL) {
-					set_clipboard_text(di->cdata);
-					break;	/* We don't need to redraw */
-				}
-				/* FIXME -- why keyboard shortcuts with shift don't works??? */
-				/* Cut to clipboard */
-				if (ev->x == KBD_DEL && ev->y == KBD_SHIFT || upcase(ev->x) == 'X' && ev->y == KBD_CTRL) {
-					set_clipboard_text(di->cdata);
-					di->cdata[0] = 0;
-					di->cpos = 0;
-					goto dsp_f;
-				}
-				/* Paste from clipboard */
-				if (ev->x == KBD_INS && ev->y == KBD_SHIFT || upcase(ev->x) == 'V' && ev->y == KBD_CTRL) {
-					char * clipboard = get_clipboard_text();
-					strncpy(di->cdata , clipboard, di->item->dlen);
-					di->cdata[di->item->dlen - 1] = 0;
-					di->cpos = strlen(di->cdata);
-					mem_free(clipboard);
-					goto dsp_f;
+					case ACT_AUTO_COMPLETE: 
+						do_tab_compl(term, &di->history, win);
+						goto dsp_f;
+					default:
+						if (ev->x >= ' ' && ev->x < 0x100 && !ev->y) {
+							if (strlen(di->cdata) < di->item->dlen - 1) {
+								memmove(di->cdata + di->cpos + 1, di->cdata + di->cpos, strlen(di->cdata) - di->cpos + 1);
+								di->cdata[di->cpos++] = ev->x;
+							}
+							goto dsp_f;
+						}
 				}
 				goto gh;
 				dsp_f:
@@ -795,7 +855,7 @@ int check_dialog(struct dialog_data *dlg)
 {
 	int i;
 	for (i = 0; i < dlg->n; i++)
-		if (dlg->dlg->items[i].type == D_CHECKBOX || dlg->dlg->items[i].type == D_FIELD)
+		if (dlg->dlg->items[i].type == D_CHECKBOX || dlg->dlg->items[i].type == D_FIELD || dlg->dlg->items[i].type == D_FIELD_PASS)
 			if (dlg->dlg->items[i].fn && dlg->dlg->items[i].fn(dlg, &dlg->items[i])) {
 				dlg->selected = i;
 				redraw_dialog(dlg);
@@ -980,7 +1040,7 @@ void max_group_width(struct terminal *term, unsigned char **texts, struct dialog
 {
 	int ww = 0;
 	while (n--) {
-		int wx = item->item->type == D_CHECKBOX ? 4 : item->item->dlen + 1;
+		int wx = item->item->type == D_CHECKBOX ? 4 : item->item->type == D_BUTTON ? strlen(_(item->item->text, term)) + 5 : item->item->dlen + 1;
 		wx += strlen(_(texts[0], term));
 		if (n) wx++;
 		ww += wx;
@@ -993,7 +1053,7 @@ void max_group_width(struct terminal *term, unsigned char **texts, struct dialog
 void min_group_width(struct terminal *term, unsigned char **texts, struct dialog_item_data *item, int n, int *w)
 {
 	while (n--) {
-		int wx = item->item->type == D_CHECKBOX ? 4 : item->item->dlen + 1;
+		int wx = item->item->type == D_CHECKBOX ? 4 : item->item->type == D_BUTTON ? strlen(_(item->item->text, term)) + 5 : item->item->dlen + 1;
 		wx += strlen(_(texts[0], term));
 		if (wx > *w) *w = wx;
 		texts++;
@@ -1018,7 +1078,7 @@ void dlg_format_group(struct terminal *term, struct terminal *t2, unsigned char
 			print_text(term, x + nx + 4 * (item->item->type == D_CHECKBOX), *y, strlen(_(texts[0], t2)), _(texts[0], t2), COLOR_DIALOG_TEXT);
 			item->x = x + nx + (sl + 1) * (item->item->type != D_CHECKBOX);
 			item->y = *y;
-			if (item->item->type == D_FIELD) item->l = item->item->dlen;
+			if (item->item->type == D_FIELD || item->item->type == D_FIELD_PASS) item->l = item->item->dlen;
 		}
 		if (rw && nx + wx > *rw) if ((*rw = nx + wx) > w) *rw = w;
 		nx += wx + 1;
@@ -1323,12 +1383,9 @@ void box_sel_set_visible(struct dialog_item_data *box_item_data, int offset) {
 	int sel;
 	
 	box = (struct dlg_data_item_data_box *)(box_item_data->item->data);
-#ifdef DEBUG		
 	if (offset > box_item_data->item->gid || offset < 0) {
-		debug("Invalid select offset: %d", offset);
 		return;
 	}
-#endif
 	/*debug("offset: %d", offset);*/
 	sel = box->box_top + offset;
 	if (sel > box->list_len) {
