@@ -53,7 +53,7 @@ void read_bookmarks() {
 			continue;
 		*url = '\0';
 		url++;
-		*( strchr(url, '\n') ) = '\0';
+		if (strchr(url, '\n')) *( strchr(url, '\n') ) = '\0';
 		add_bookmark(title, url);
 	}
 	
@@ -75,10 +75,14 @@ void write_bookmarks() {
 	if (!out) return;
 
 	foreachback(bm, bookmarks) {
-		fputs(bm->title,out);
+		unsigned char *p = stracpy(bm->title);
+		int i;
+		for (i = strlen(p) - 1; i >= 0; i--) if (p[i] < ' '|| p[i] == '|') p[i] = '*';
+		fputs(p,out);
 		fputc('|', out);
 		fputs(bm->url,out);
 		fputc('\n',out);
+		mem_free(p);
 	}
 
 	fclose(out);
