@@ -272,6 +272,9 @@ void delete_cache_entry(struct cache_entry *e)
 	if (e->head) mem_free(e->head);
 	if (e->last_modified) mem_free(e->last_modified);
 	if (e->redirect) mem_free(e->redirect);
+#ifdef HAVE_SSL
+	if (e->ssl_info) mem_free(e->ssl_info);
+#endif
 	mem_free(e);
 }
 
