@@ -20,19 +20,8 @@ void get_system_name()
 	strcpy(system_name, SYSTEM_NAME);
 }
 
-struct option {
-	int p;
-	unsigned char *(*rd_cmd)(struct option *, unsigned char ***, int *);
-	unsigned char *(*rd_cfg)(struct option *, unsigned char *);
-	void (*wr_cfg)(struct option *, unsigned char **, int *);
-	int min, max;
-	void *ptr;
-	unsigned char *cfg_name;
-	unsigned char *cmd_name;
-};
-
-struct option links_options[];
-struct option html_options[];
+extern struct option links_options[];
+extern struct option html_options[];
 
 struct option *all_options[] = { links_options, html_options, NULL, };
 
@@ -70,13 +59,44 @@ unsigned char *parse_options(int argc, unsigned char *argv[])
 	return _parse_options(argc, argv, all_options);
 }
 
-void parse_config_file(unsigned char *name, unsigned char *file, struct option *options)
+unsigned char *get_token(unsigned char **line)
 {
+	unsigned char *s = NULL;
+	int l = 0;
+	int escape = 0;
+	int quote = 0;
+	
+	while (**line == ' ' || **line == 9) (*line)++;
+	if (**line) {
+		for (s = init_str(); **line; (*line)++) {
+			if (escape) 
+				escape = 0;
+			else if (**line == '\\') {
+				escape = 1; 
+				continue;
+			}	
+			else if (**line == '"') {
+				quote = !quote;
+			    	continue;
+			}
+			else if ((**line == ' ' || **line == 9) && !quote)
+				break;
+			add_chr_to_str(&s, &l, **line);
+		}
+	}
+	return s;
+}
+
+void parse_config_file(unsigned char *name, unsigned char *file, struct option **opt)
+{
+	struct option *options;
+	struct option **op;
 	int err = 0;
 	int line = 0;
 	unsigned char *e;
 	int i;
 	unsigned char *n, *p;
+	unsigned char *tok;
 	int nl, pl;
 	while (file[0]) {
 		line++;
@@ -96,17 +116,23 @@ void parse_config_file(unsigned char *name, unsigned char *file, struct option *
 			if ((file[1] == 10 || file[1] == 13) && file[0] != file[1]) file++;
 			file++;
 		}
-		if (n[0] != '#') for (i = 0; options[i].p; i++) if (options[i].cfg_name && nl == strlen(options[i].cfg_name) && !casecmp(n, options[i].cfg_name, nl)) {
-			unsigned char *o = memacpy(p, pl);
-			if ((e = options[i].rd_cfg(&options[i], o))) {
-				if (e[0]) fprintf(stderr, "Error parsing config file %s, line %d: %s\n", name, line, e), err = 1;
+		tok = NULL;
+		if (n[0] == '#') goto f;
+		if (!(tok = get_token(&n))) goto f;
+		nl = strlen(tok);
+		for (op = opt; (options = *op); op++) 
+		    	for (i = 0; options[i].p; i++) if (options[i].cfg_name && nl == strlen(options[i].cfg_name) && !casecmp(tok, options[i].cfg_name, nl)) {
+				unsigned char *o = memacpy(p, pl);
+				if ((e = options[i].rd_cfg(&options[i], o))) {
+					if (e[0]) fprintf(stderr, "Error parsing config file %s, line %d: %s\n", name, line, e), err = 1;
+				}
+				mem_free(o);
+				goto f;
 			}
-			mem_free(o);
-			goto f;
-		}
 		fprintf(stderr, "Unknown option in config file %s, line %d\n", name, line);
 		err = 1;
-		f:;
+		f:
+		if (tok) mem_free(tok);
 	}
 	if (err) fprintf(stderr, "\007"), sleep(3);
 }
@@ -116,6 +142,7 @@ unsigned char *create_config_string(struct option *options)
 	unsigned char *s = init_str();
 	int l = 0;
 	int i;
+	add_to_str(&s, &l, "# This file is automatically generated by Links -- please do not edit.");
 	for (i = 0; options[i].p; i++) if (options[i].wr_cfg)
 		options[i].wr_cfg(&options[i], &s, &l);
 	add_to_str(&s, &l, NEWLINE);
@@ -166,7 +193,7 @@ unsigned char *get_home(int *n)
 {
 	struct stat st;
 	unsigned char *home = stracpy(getenv("HOME"));
-	unsigned char *links_home;
+	unsigned char *home_links;
 	unsigned char *config_dir = stracpy(getenv("CONFIG_DIR"));
 
 	if (n) *n = 1;
@@ -179,53 +206,58 @@ unsigned char *get_home(int *n)
 		}
 		for (i = strlen(home) - 1; i >= 0; i--) if (dir_sep(home[i])) {
 			home[i + 1] = 0;
-			break;
+			goto br;
 		}
+		home[0] = 0;
+		br:;
 	}
 	while (home[0] && dir_sep(home[strlen(home) - 1])) home[strlen(home) - 1] = 0;
-	add_to_strn(&home, "/");
-	links_home = stracpy(home);
+	if (home[0]) add_to_strn(&home, "/");
+	home_links = stracpy(home);
 	if (config_dir) 	
 	{
-		add_to_strn(&links_home,config_dir);
-		while (links_home[0] && dir_sep(links_home[strlen(links_home) - 1])) links_home[strlen(links_home) - 1] = 0;
-		if (stat(links_home,&st) != -1 && S_ISDIR(st.st_mode)) {
-			add_to_strn(&links_home, "/links");
+		add_to_strn(&home_links, config_dir);
+		while (home_links[0] && dir_sep(home_links[strlen(home_links) - 1])) home_links[strlen(home_links) - 1] = 0;
+		if (stat(home_links, &st) != -1 && S_ISDIR(st.st_mode)) {
+			add_to_strn(&home_links, "/links");
 	    	} else {
-			fprintf(stderr, "CONFIG_DIR set to %s. But directory %s doesn't egzist.", config_dir, links_home);
-			mem_free(links_home);
-			links_home = stracpy(home);
-			add_to_strn(&links_home, ".links");		
+			fprintf(stderr, "CONFIG_DIR set to %s. But directory %s doesn't exist.\n\007", config_dir, home_links);
+			sleep(3);
+			mem_free(home_links);
+			home_links = stracpy(home);
+			add_to_strn(&home_links, ".links");		
 		}
 		mem_free(config_dir);
-	} else add_to_strn(&links_home, ".links");
-	if (stat(links_home, &st)) {
-		if (!mkdir(links_home, 0777)) goto home_creat;
+	} else add_to_strn(&home_links, ".links");
+	if (stat(home_links, &st)) {
+		if (!mkdir(home_links, 0777)) goto home_creat;
+		if (config_dir) goto failed;
 		goto first_failed;
 	}
 	if (S_ISDIR(st.st_mode)) goto home_ok;
 	first_failed:
-	mem_free(links_home);
-	links_home = stracpy(home);
-	add_to_strn(&links_home, "links");
-	if (stat(links_home, &st)) {
-		if (!mkdir(links_home, 0777)) goto home_creat;
+	mem_free(home_links);
+	home_links = stracpy(home);
+	add_to_strn(&home_links, "links");
+	if (stat(home_links, &st)) {
+		if (!mkdir(home_links, 0777)) goto home_creat;
 		goto failed;
 	}
 	if (S_ISDIR(st.st_mode)) goto home_ok;
 	failed:
-	mem_free(links_home);
+	mem_free(home_links);
 	mem_free(home);
 	return NULL;
+
 	home_ok:
 	if (n) *n = 0;
 	home_creat:
 #ifdef HAVE_CHMOD
-	chmod(links_home, 0700);
+	chmod(home_links, 0700);
 #endif
-	add_to_strn(&links_home, "/");
+	add_to_strn(&home_links, "/");
 	mem_free(home);
-	return links_home;
+	return home_links;
 }
 
 void init_home()
@@ -233,46 +265,46 @@ void init_home()
 	get_system_name();
 	links_home = get_home(&first_use);
 	if (!links_home) {
-		fprintf(stderr, "%s\n\007", "Unable to find or create links config directory. Please check, that you have $HOME variable set correctly and that you have write permission to your home directory");
+		fprintf(stderr, "Unable to find or create links config directory. Please check, that you have $HOME variable set correctly and that you have write permission to your home directory.\n\007");
 		sleep(3);
 		return;
 	}
 }
 
-void load_config_file(unsigned char *name, struct option *o)
+void load_config_file(unsigned char *prefix, unsigned char *name)
 {
 	unsigned char *c, *config_file;
-	config_file = stracpy(links_home);
+	config_file = stracpy(prefix);
 	if (!config_file) return;
 	add_to_strn(&config_file, name);
 	if ((c = read_config_file(config_file))) goto ok;
 	mem_free(config_file);
-	config_file = stracpy(links_home);
+	config_file = stracpy(prefix);
 	if (!config_file) return;
 	add_to_strn(&config_file, ".");
 	add_to_strn(&config_file, name);
 	if ((c = read_config_file(config_file))) goto ok;
 	mem_free(config_file);
-	/*fprintf(stderr, "%s\n\007", "Unable to find or create config file in your links directory. Please check, that you have write permission to links home directory");
-	sleep(3);*/
 	return;
 	ok:
-	parse_config_file(config_file, c, o);
+	parse_config_file(config_file, c, all_options);
 	mem_free(c);
 	mem_free(config_file);
 }
 
 void load_config()
 {
-	load_config_file("links.cfg", links_options);
-	load_config_file("html.cfg", html_options);
+	load_config_file("/etc/", "links.cfg");
+	load_config_file(links_home, "links.cfg");
+	load_config_file(links_home, "html.cfg");
+	load_config_file(links_home, "user.cfg");
 }
 
-int write_config_file(unsigned char *name, struct option *o, struct terminal *term)
+int write_config_file(unsigned char *prefix, unsigned char *name, struct option *o, struct terminal *term)
 {
 	unsigned char *c, *config_file;
 	if (!(c = create_config_string(o))) return -1;
-	config_file = stracpy(links_home);
+	config_file = stracpy(prefix);
 	if (!config_file) {
 		mem_free(c);
 		return -1;
@@ -291,12 +323,12 @@ int write_config_file(unsigned char *name, struct option *o, struct terminal *te
 
 void write_config(struct terminal *term)
 {
-	write_config_file("links.cfg", links_options, term);
+	write_config_file(links_home, "links.cfg", links_options, term);
 }
 
 void write_html_config(struct terminal *term)
 {
-	write_config_file("html.cfg", html_options, term);
+	write_config_file(links_home, "html.cfg", html_options, term);
 }
 
 void add_nm(struct option *o, unsigned char **s, int *l)
@@ -306,13 +338,34 @@ void add_nm(struct option *o, unsigned char **s, int *l)
 	add_to_str(s, l, " ");
 }
 
+void add_quoted_to_str(unsigned char **s, int *l, unsigned char *q)
+{
+	add_chr_to_str(s, l, '"');
+	while (*q) {
+		if (*q == '"' || *q == '\\') add_chr_to_str(s, l, '\\');
+		add_chr_to_str(s, l, *q);
+		q++;
+	}
+	add_chr_to_str(s, l, '"');
+}
+
 unsigned char *num_rd(struct option *o, unsigned char *c)
 {
+	unsigned char *tok = get_token(&c);
 	unsigned char *end;
-	long l = strtolx(c, &end);
-	if (*end) return "Number expected";
-	if (l < o->min || l > o->max) return "Out of range";
+	long l;
+	if (!tok) return "Missing argument";
+	l = strtolx(tok, &end);
+	if (*end) {
+		mem_free(tok);
+		return "Number expected";
+	}
+	if (l < o->min || l > o->max) {
+		mem_free(tok);
+		return "Out of range";
+	}
 	*(int *)o->ptr = l;
+	mem_free(tok);
 	return NULL;
 }
 
@@ -324,25 +377,39 @@ void num_wr(struct option *o, unsigned char **s, int *l)
 
 unsigned char *str_rd(struct option *o, unsigned char *c)
 {
-	if (strlen(c) + 1 > o->max) return "String too long";
-	strcpy(o->ptr, c);
-	return NULL;
+	unsigned char *tok = get_token(&c);
+	unsigned char *e = NULL;
+	if (!tok) return NULL;
+	if (strlen(tok) + 1 > o->max) e = "String too long";
+	else strcpy(o->ptr, tok);
+	mem_free(tok);
+	return e;
 }
 
 void str_wr(struct option *o, unsigned char **s, int *l)
 {
 	add_nm(o, s, l);
-	if (strlen(o->ptr) > o->max - 1) add_bytes_to_str(s, l, o->ptr, o->max - 1);
-	else add_to_str(s, l, o->ptr);
+	if (strlen(o->ptr) > o->max - 1) {
+		unsigned char *s1 = init_str();
+		int l1 = 0;
+		add_bytes_to_str(&s1, &l1, o->ptr, o->max - 1);
+		add_quoted_to_str(s, l, s1);
+		mem_free(s1);
+	}
+	else add_quoted_to_str(s, l, o->ptr);
 }
 
 unsigned char *cp_rd(struct option *o, unsigned char *c)
 {
+	unsigned char *tok = get_token(&c);
+	unsigned char *e = NULL;
 	int i;
+	if (!tok) return "Missing argument";
 	/*if (!strcasecmp(c, "none")) i = -1;
-	else */if ((i = get_cp_index(c)) == -1) return "Unknown codepage";
-	*(int *)o->ptr = i;
-	return NULL;
+	else */if ((i = get_cp_index(tok)) == -1) e = "Unknown codepage";
+	else *(int *)o->ptr = i;
+	mem_free(tok);
+	return e;
 }
 
 void cp_wr(struct option *o, unsigned char **s, int *l)
@@ -355,59 +422,22 @@ void cp_wr(struct option *o, unsigned char **s, int *l)
 unsigned char *lang_rd(struct option *o, unsigned char *c)
 {
 	int i;
+	unsigned char *tok = get_token(&c);
+	if (!tok) return "Missing argument";
 	for (i = 0; i < n_languages(); i++)
-		if (!(strcasecmp(language_name(i), c))) {
+		if (!(strcasecmp(language_name(i), tok))) {
 			set_language(i);
+			mem_free(tok);
 			return NULL;
 		}
+	mem_free(tok);
 	return "Unknown language";
 }
 
 void lang_wr(struct option *o, unsigned char **s, int *l)
 {
 	add_nm(o, s, l);
-	add_to_str(s, l, language_name(current_language));
-}
-
-unsigned char *get_word(unsigned char **s)
-{
-	unsigned char *w = *s;
-	unsigned char *ww;
-	if (!**s) return NULL;
-	while (**s > ' ') (*s)++;
-	ww = memacpy(w, *s - w);
-	while (**s && **s <= ' ') (*s)++;
-	return ww;
-}
-
-unsigned char *get_quoted(unsigned char **s)
-{
-	unsigned char *st;
-	int l;
-	if (!**s || **s != '"') return NULL;
-	(*s)++;
-	st = init_str();
-	l = 0;
-	while (**s && **s != '"') quak:add_chr_to_str(&st, &l, *((*s)++));
-	if (!**s) {
-		mem_free(st);
-		return NULL;
-	}
-	(*s)++;
-	if (**s == '"') goto quak;
-	while (**s && **s <= ' ') (*s)++;
-	return st;
-}
-
-void add_quoted_to_str(unsigned char **s, int *l, unsigned char *q)
-{
-	add_chr_to_str(s, l, '"');
-	while (*q) {
-		if (*q == '"') add_to_str(s, l, "\"\"");
-		else add_chr_to_str(s, l, *q);
-		q++;
-	}
-	add_chr_to_str(s, l, '"');
+	add_quoted_to_str(s, l, language_name(current_language));
 }
 
 int getnum(unsigned char *s, int *n, int r1, int r2)
@@ -427,10 +457,10 @@ unsigned char *type_rd(struct option *o, unsigned char *c)
 	unsigned char *w;
 	int n;
 	memset(&new, 0, sizeof(struct assoc));
-	if (!(new.label = get_quoted(&c))) goto err;
-	if (!(new.ct = get_quoted(&c))) goto err;
-	if (!(new.prog = get_quoted(&c))) goto err;
-	if (!(w = get_word(&c))) goto err;
+	if (!(new.label = get_token(&c))) goto err;
+	if (!(new.ct = get_token(&c))) goto err;
+	if (!(new.prog = get_token(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if (getnum(w, &n, 0, 32)) goto err_f;
 	mem_free(w);
 	new.cons = !!(n & 1);
@@ -438,7 +468,7 @@ unsigned char *type_rd(struct option *o, unsigned char *c)
 	new.ask = !!(n & 4);
 	if ((n & 8) || (n & 16)) new.block = !!(n & 16);
 	else new.block = !new.xwin || new.cons;
-	if (!(w = get_word(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if (strlen(w) != 1 || w[0] < '0' || w[0] > '9') goto err_f;
 	new.system = w[0] - '0';
 	mem_free(w);
@@ -454,7 +484,6 @@ unsigned char *type_rd(struct option *o, unsigned char *c)
 	goto err;
 }
 
-
 void type_wr(struct option *o, unsigned char **s, int *l)
 {
 	struct assoc *a;
@@ -477,8 +506,8 @@ unsigned char *ext_rd(struct option *o, unsigned char *c)
 	unsigned char *err = "Error reading extension specification";
 	struct extension new;
 	memset(&new, 0, sizeof(struct extension));
-	if (!(new.ext = get_quoted(&c))) goto err;
-	if (!(new.ct = get_quoted(&c))) goto err;
+	if (!(new.ext = get_token(&c))) goto err;
+	if (!(new.ct = get_token(&c))) goto err;
 	update_ext(&new);
 	err = NULL;
 	err:
@@ -502,8 +531,8 @@ unsigned char *prog_rd(struct option *o, unsigned char *c)
 {
 	unsigned char *err = "Error reading program specification";
 	unsigned char *prog, *w;
-	if (!(prog = get_quoted(&c))) goto err_1;
-	if (!(w = get_word(&c))) goto err_2;
+	if (!(prog = get_token(&c))) goto err_1;
+	if (!(w = get_token(&c))) goto err_2;
 	if (strlen(w) != 1 || w[0] < '0' || w[0] > '9') goto err_3;
 	update_prog(o->ptr, prog, w[0] - '0');
 	err = NULL;
@@ -519,6 +548,7 @@ void prog_wr(struct option *o, unsigned char **s, int *l)
 {
 	struct protocol_program *a;
 	foreachback(a, *(struct list_head *)o->ptr) {
+		if (!*a->prog) continue;
 		add_nm(o, s, l);
 		add_quoted_to_str(s, l, a->prog);
 		add_to_str(s, l, " ");
@@ -531,27 +561,27 @@ unsigned char *term_rd(struct option *o, unsigned char *c)
 	struct term_spec *ts;
 	unsigned char *w;
 	int i;
-	if (!(w = get_quoted(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if (!(ts = new_term_spec(w))) {
 		mem_free(w);
 		goto end;
 	}
 	mem_free(w);
-	if (!(w = get_word(&c))) goto err;
-	if (strlen(w) != 1 || w[0] < '0' || w[0] > '2') goto err_f;
+	if (!(w = get_token(&c))) goto err;
+	if (strlen(w) != 1 || w[0] < '0' || w[0] > '3') goto err_f;
 	ts->mode = w[0] - '0';
 	mem_free(w);
-	if (!(w = get_word(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if (strlen(w) != 1 || w[0] < '0' || w[0] > '1') goto err_f;
 	ts->m11_hack = w[0] - '0';
 	mem_free(w);
-	if (!(w = get_word(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if (strlen(w) != 1 || w[0] < '0' || w[0] > '7') goto err_f;
 	ts->col = (w[0] - '0') & 1;
 	ts->restrict_852 = !!((w[0] - '0') & 2);
 	ts->block_cursor = !!((w[0] - '0') & 4);
 	mem_free(w);
-	if (!(w = get_word(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if ((i = get_cp_index(w)) == -1) goto err_f;
 	ts->charset = i;
 	mem_free(w);
@@ -568,29 +598,29 @@ unsigned char *term2_rd(struct option *o, unsigned char *c)
 	struct term_spec *ts;
 	unsigned char *w;
 	int i;
-	if (!(w = get_quoted(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if (!(ts = new_term_spec(w))) {
 		mem_free(w);
 		goto end;
 	}
 	mem_free(w);
-	if (!(w = get_word(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if (strlen(w) != 1 || w[0] < '0' || w[0] > '3') goto err_f;
 	ts->mode = w[0] - '0';
 	mem_free(w);
-	if (!(w = get_word(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if (strlen(w) != 1 || w[0] < '0' || w[0] > '1') goto err_f;
 	ts->m11_hack = w[0] - '0';
 	mem_free(w);
-	if (!(w = get_word(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if (strlen(w) != 1 || w[0] < '0' || w[0] > '1') goto err_f;
 	ts->restrict_852 = w[0] - '0';
 	mem_free(w);
-	if (!(w = get_word(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if (strlen(w) != 1 || w[0] < '0' || w[0] > '1') goto err_f;
 	ts->col = w[0] - '0';
 	mem_free(w);
-	if (!(w = get_word(&c))) goto err;
+	if (!(w = get_token(&c))) goto err;
 	if ((i = get_cp_index(w)) == -1) goto err_f;
 	ts->charset = i;
 	mem_free(w);
@@ -738,10 +768,17 @@ Options are:\n\
   No local file browsing. No downloads. Executing of viewers\n\
   is allowed, but user can't add or modify entries in\n\
   association table.\n\
+\n\
+ -dump\n\
+  Write a plain-text version of the given HTML document to\n\
+  stdout.\n\
 \n\
  -no-connect\n\
   Runs links as a separate instance - instead of connecting to\n\
   existing instance.\n\
+\n\
+ -source\n\
+  Write the given HTML document in source form to stdout.\n\
 \n\
  -version\n\
   Prints the links version number and exit.\n\
@@ -860,7 +897,7 @@ struct option links_options[] = {
 	1, gen_cmd, num_rd, num_wr, 0, 1, &http_bugs.bug_302_redirect, "http_bugs.bug_302_redirect", "http-bugs.bug-302-redirect",
 	1, gen_cmd, num_rd, num_wr, 0, 1, &http_bugs.bug_post_no_keepalive, "http_bugs.bug_post_no_keepalive", "http-bugs.bug_post-no-keepalive",
 	1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, default_anon_pass, "ftp.anonymous_password", "ftp.anonymous-password",
-	1, NULL, cp_rd, NULL, 0, 0, &dds.assume_cp, "assume_codepage", "assume-codepage",
+	1, gen_cmd, cp_rd, NULL, 0, 0, &dds.assume_cp, "assume_codepage", "assume-codepage",
 	1, NULL, term_rd, term_wr, 0, 0, NULL, "terminal", NULL,
 	1, NULL, term2_rd, NULL, 0, 0, NULL, "terminal2", NULL,
 	1, NULL, type_rd, type_wr, 0, 0, NULL, "association", NULL,
@@ -868,10 +905,13 @@ struct option links_options[] = {
 	1, NULL, prog_rd, prog_wr, 0, 0, &mailto_prog, "mailto", NULL,
 	1, NULL, prog_rd, prog_wr, 0, 0, &telnet_prog, "telnet", NULL,
 	1, NULL, prog_rd, prog_wr, 0, 0, &tn3270_prog, "tn3270", NULL,
+	1, NULL, bind_rd, NULL, 0, 0, NULL, "bind", NULL,
+	1, NULL, unbind_rd, NULL, 0, 0, NULL, "unbind", NULL,
 	0, NULL, NULL, NULL, 0, 0, NULL, NULL, NULL,
 };
 
 struct option html_options[] = {
+	1, gen_cmd, num_rd, num_wr, 0, 1, &dds.hard_assume, "html_hard_assume", "html-hard-assume",
 	1, gen_cmd, cp_rd, cp_wr, 0, 0, &dds.assume_cp, "html_assume_codepage", "html-assume-codepage",
 	1, gen_cmd, num_rd, num_wr, 0, 1, &dds.tables, "html_tables", "html-tables",
 	1, gen_cmd, num_rd, num_wr, 0, 1, &dds.frames, "html_frames", "html-frames",
