@@ -123,6 +123,10 @@ static inline int fg_color(int fg, int bg)
 	return fg;
 }
 
+#ifdef ALIGN
+#undef ALIGN
+#endif
+
 #define ALIGN(x) (((x)+0x7f)&~0x7f)
 
 static inline int xpand_lines(struct part *p, int y)
@@ -862,17 +866,22 @@ struct conv_table *get_convert_table(unsigned char *head, int to, int def, int *
 {
 	int from = -1;
 	unsigned char *a, *b;
-	if (head && (a = parse_http_header(head, "Content-Charset", NULL))) {
-		from = get_cp_index(a);
-		mem_free(a);
-	}
-	if (from == -1 && head && (a = parse_http_header(head, "Content-Type", NULL))) {
+	unsigned char *p = head;
+	while (from == -1 && p && (a = parse_http_header(p, "Content-Type", &p))) {
 		if ((b = parse_header_param(a, "charset"))) {
 			from = get_cp_index(b);
 			mem_free(b);
 		}
 		mem_free(a);
 	}
+	if (from == -1 && head && (a = parse_http_header(head, "Content-Charset", NULL))) {
+		from = get_cp_index(a);
+		mem_free(a);
+	}
+	if (from == -1 && head && (a = parse_http_header(head, "Charset", NULL))) {
+		from = get_cp_index(a);
+		mem_free(a);
+	}
 	if (aa) {
 		*aa = from == -1;
 		if (hard && !*aa) *aa = 2;
