@@ -32,6 +32,7 @@ unsigned char *parse_http_header(unsigned char *head, unsigned char *item, unsig
 					*ptr = f;
 					break;
 				}
+				return h;
 			}
 		}
 		cont:;
@@ -339,7 +340,7 @@ void read_http_data(struct connection *c, struct read_buffer *rb)
 		if (info->length >= 0 && info->length < l) l = info->length;
 		c->received += l;
 		if (add_fragment(c->cache, c->from, rb->data, l) == 1) c->tries = 0;
-		info->length -= l;
+		if (info->length >= 0) info->length -= l;
 		c->from += l;
 		kill_buffer_data(rb, l);
 		if (!info->length && !rb->close) {
@@ -430,6 +431,7 @@ int get_header(struct read_buffer *rb)
 
 void http_got_header(struct connection *c, struct read_buffer *rb)
 {
+	int cf;
 	int state = c->state != S_PROC ? S_GETH : S_PROC;
 	unsigned char *head;
 	unsigned char *cookie, *ch;
@@ -557,6 +559,7 @@ void http_got_header(struct connection *c, struct read_buffer *rb)
 		if (!strcasecmp(d, "close")) info->close = 1;
 		mem_free(d);
 	} else if (version < 11) info->close = 1;
+	cf = c->from;
 	c->from = 0;
 	if ((d = parse_http_header(e->head, "Content-Range", NULL))) {
 		if (strlen(d) > 6) {
@@ -568,6 +571,12 @@ void http_got_header(struct connection *c, struct read_buffer *rb)
 		}
 		mem_free(d);
 	}
+	if (cf && !c->from && !c->unrestartable) c->unrestartable = 1;
+	if (c->from > cf || c->from < 0) {
+		setcstate(c, S_HTTP_ERROR);
+		abort_connection(c);
+		return;
+	}
 	if ((d = parse_http_header(e->head, "Content-Length", NULL))) {
 		unsigned char *ep;
 		int l = strtol(d, (char **)&ep, 10);
@@ -577,8 +586,12 @@ void http_got_header(struct connection *c, struct read_buffer *rb)
 		}
 		mem_free(d);
 	}
-	if ((d = parse_http_header(e->head, "Accept-Ranges", NULL))) mem_free(d);
-	else if (!c->unrestartable) c->unrestartable = 1;
+	if ((d = parse_http_header(e->head, "Accept-Ranges", NULL))) {
+		if (!strcasecmp(d, "none")) {
+			if (!c->unrestartable) c->unrestartable = 1;
+		}
+		mem_free(d);
+	} else if (!c->unrestartable && !c->from) c->unrestartable = 1;
 	if ((d = parse_http_header(e->head, "Transfer-Encoding", NULL))) {
 		if (!strcasecmp(d, "chunked")) {
 			info->length = -2;
