@@ -3,6 +3,11 @@
 #define OUT_BUF_SIZE	16384
 #define IN_BUF_SIZE	16
 
+#define USE_TWIN_MOUSE	1
+#define TW_BUTT_LEFT	1
+#define TW_BUTT_MIDDLE	2
+#define TW_BUTT_RIGHT	4
+
 struct itrm {
 	int std_in;
 	int std_out;
@@ -11,6 +16,7 @@ struct itrm {
 	int ctl_in;
 	int blocked;
 	struct termios t;
+	int flags;
 	unsigned char kqueue[IN_BUF_SIZE];
 	int qlen;
 	int tm;
@@ -81,18 +87,33 @@ unsigned char *init_seq = "\033[?1000h\033[?47h\0337";
 unsigned char *term_seq = "\033[2J\033[?1000l\033[?47l\0338\b \b";
 */
 
-unsigned char *init_seq = "\033)0\0337\033[?1000h";
-unsigned char *term_seq = "\033[?1000l\033[2J\0338\r \b";
+unsigned char *init_seq = "\033)0\0337";
+unsigned char *init_seq_x_mouse = "\033[?1000h";
+unsigned char *init_seq_tw_mouse = "\033[?9h";
+unsigned char *term_seq = "\033[2J\0338\r \b";
+unsigned char *term_seq_x_mouse = "\033[?1000l";
+unsigned char *term_seq_tw_mouse = "\033[?9l";
+
 /*unsigned char *term_seq = "\033[2J\033[?1000l\0338\b \b";*/
 
-void send_init_sequence(int h)
+void send_init_sequence(int h,int flags)
 {
 	hard_write(h, init_seq, strlen(init_seq));
+	if (flags & USE_TWIN_MOUSE) {
+		hard_write(h, init_seq_tw_mouse, strlen(init_seq_tw_mouse));
+	} else {
+		hard_write(h, init_seq_x_mouse, strlen(init_seq_x_mouse));
+	}
 }
 
-void send_term_sequence(int h)
+void send_term_sequence(int h,int flags)
 {
 	hard_write(h, term_seq, strlen(term_seq));
+	if (flags & USE_TWIN_MOUSE) {
+		hard_write(h, term_seq_tw_mouse, strlen(term_seq_tw_mouse));
+	} else {
+		hard_write(h, term_seq_x_mouse, strlen(term_seq_x_mouse));
+	}
 }
 
 void resize_terminal()
@@ -151,7 +172,10 @@ void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in,
 	ev.y = y;
 	handle_terminal_resize(ctl_in, resize_terminal);
 	queue_event(itrm, (char *)&ev, sizeof(struct event));
+	xwin = is_xterm() * ENV_XWIN + can_twterm() * ENV_TWIN + (!!getenv("STY")) * ENV_SCREEN + get_system_env();
+	itrm->flags = 0;
 	if (!(ts = getenv("TERM"))) ts = "";
+	if ((xwin & ENV_TWIN) && !strcmp(ts,"linux")) itrm->flags |= USE_TWIN_MOUSE;
 	if (strlen(ts) >= MAX_TERM_LEN) queue_event(itrm, ts, MAX_TERM_LEN);
 	else {
 		unsigned char *mm;
@@ -181,14 +205,13 @@ void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in,
 	}
 	mem_free(ts);
 	neni_pamet:
-	xwin = is_xterm() * ENV_XWIN + (!!getenv("STY")) * ENV_SCREEN + get_system_env();
 	queue_event(itrm, (char *)&xwin, sizeof(int));
 	queue_event(itrm, (char *)&init_len, sizeof(int));
 	queue_event(itrm, (char *)init_string, init_len);
 	itrm->mouse_h = handle_mouse(0, (void (*)(void *, unsigned char *, int))queue_event, itrm);
 	itrm->orig_title = get_window_title();
 	set_window_title("Links");
-	send_init_sequence(std_out);
+	send_init_sequence(std_out,itrm->flags);
 }
 
 void unblock_itrm_x(void *h)
@@ -209,7 +232,7 @@ int unblock_itrm(int fd)
 	}*/
 	if (itrm->ctl_in >= 0 && setraw(itrm->ctl_in, NULL)) return -1;
 	itrm->blocked = 0;
-	send_init_sequence(itrm->std_out);
+	send_init_sequence(itrm->std_out,itrm->flags);
 	set_handlers(itrm->std_in, (void (*)(void *))in_kbd, NULL, (void (*)(void *))free_trm, itrm);
 	handle_terminal_resize(itrm->ctl_in, resize_terminal);
 	unblock_stdin();
@@ -227,7 +250,7 @@ void block_itrm(int fd)
 	itrm->blocked = 1;
 	block_stdin();
 	unhandle_terminal_resize(itrm->ctl_in);
-	send_term_sequence(itrm->std_out);
+	send_term_sequence(itrm->std_out,itrm->flags);
 	tcsetattr(itrm->ctl_in, TCSANOW, &itrm->t);
 	set_handlers(itrm->std_in, NULL, NULL, (void (*)(void *))free_trm, itrm);
 }
@@ -238,7 +261,7 @@ void free_trm(struct itrm *itrm)
 	set_window_title(itrm->orig_title);
 	if (itrm->orig_title) mem_free(itrm->orig_title), itrm->orig_title = NULL;
 	unhandle_terminal_resize(itrm->ctl_in);
-	send_term_sequence(itrm->std_out);
+	send_term_sequence(itrm->std_out,itrm->flags);
 	tcsetattr(itrm->ctl_in, TCSANOW, &itrm->t);
 	if (itrm->mouse_h) unhandle_mouse(itrm->mouse_h);
 	set_handlers(itrm->std_in, NULL, NULL, NULL, NULL);
@@ -367,10 +390,10 @@ void kbd_timeout(struct itrm *itrm)
 {
 	struct event ev = {EV_KBD, KBD_ESC, 0, 0};
 	itrm->tm = -1;
-	if (can_read(itrm->std_in)) {
+	/*if (can_read(itrm->std_in)) {
 		in_kbd(itrm);
 		return;
-	}
+	}*/
 	if (!itrm->qlen) {
 		internal("timeout on empty queue");
 		return;
@@ -483,6 +506,9 @@ int process_queue(struct itrm *itrm)
 				case 'e': ev.x = KBD_END; break;
 				case 'H':
 				case 0: ev.x = KBD_HOME; break;
+				case 'I': ev.x = KBD_PAGE_UP; break;
+				case 'G': ev.x = KBD_PAGE_DOWN; break;
+
 				case 'z': switch (v) {
 					case 247: ev.x = KBD_INS; break;
 					case 214: ev.x = KBD_HOME; break;
@@ -498,8 +524,8 @@ int process_queue(struct itrm *itrm)
 					case 4: ev.x = KBD_END; break;
 					case 5: ev.x = KBD_PAGE_UP; break;
 					case 6: ev.x = KBD_PAGE_DOWN; break;
-					case 7: ev.x = KBD_END; break;
-					case 8: ev.x = KBD_HOME; break;
+					case 7: ev.x = KBD_HOME; break;
+					case 8: ev.x = KBD_END; break;
 					case 17: ev.x = KBD_F6; break;
 					case 18: ev.x = KBD_F7; break;
 					case 19: ev.x = KBD_F8; break;
@@ -511,19 +537,37 @@ int process_queue(struct itrm *itrm)
 				case 'R':
 						  resize_terminal (); break ;
 				case 'M': if (itrm->qlen - el < 3) goto ret;
-					ev.ev = EV_MOUSE;
-					ev.x = itrm->kqueue[el+1] - ' ' - 1;
-					ev.y = itrm->kqueue[el+2] - ' ' - 1;
-					ev.b = B_DOWN;
-					if (itrm->kqueue[el] & 4) ev.b = B_DRAG;
-					if ((ev.b |= (itrm->kqueue[el] & BM_BUTT) | B_DOWN) == 3) {
-						ev.b = B_UP;
-						if (xterm_button != -1) ev.b |= xterm_button;
+					if (v == 5) {
+						if (xterm_button == -1) xterm_button = 0; /* */
+						if (itrm->qlen - el < 5) goto ret;
+						ev.x = (unsigned char)(itrm->kqueue[el+1]) - ' ' - 1 + ((int)((unsigned char)(itrm->kqueue[el+2]) - ' ' - 1) << 7);
+						if ( ev.x & (1 << 13)) ev.x = 0; /* ev.x |= ~0 << 14; */
+						ev.y = (unsigned char)(itrm->kqueue[el+3]) - ' ' - 1 + ((int)((unsigned char)(itrm->kqueue[el+4]) - ' ' - 1) << 7);
+						if ( ev.y & (1 << 13)) ev.y = 0; /* ev.y |= ~0 << 14; */
+						switch ((itrm->kqueue[el] - ' ') ^ xterm_button) { /* Every event changhes only one bit */
+						    case TW_BUTT_LEFT:   ev.b = B_LEFT | ( (xterm_button & TW_BUTT_LEFT) ? B_UP : B_DOWN ); break; 
+						    case TW_BUTT_MIDDLE: ev.b = B_MIDDLE | ( (xterm_button & TW_BUTT_MIDDLE) ? B_UP :  B_DOWN ); break;
+						    case TW_BUTT_RIGHT:  ev.b = B_RIGHT | ( (xterm_button & TW_BUTT_RIGHT) ? B_UP : B_DOWN ); break;
+						    case 0: ev.b = B_DRAG;
+						    /* default : Twin protocol error */
+						}
+						xterm_button = itrm->kqueue[el] - ' ';
+						el += 5;
+					} else {
+						ev.x = itrm->kqueue[el+1] - ' ' - 1;
+						ev.y = itrm->kqueue[el+2] - ' ' - 1;
+						ev.b = B_DOWN;
+						if (itrm->kqueue[el] & 4) ev.b = B_DRAG;
+						if ((ev.b |= (itrm->kqueue[el] & BM_BUTT) | B_DOWN) == 3) {
+							ev.b = B_UP;
+							if (xterm_button != -1) ev.b |= xterm_button;
+						}  
+						/*if ((itrm->kqueue[el] & 4) && ev.b != B_UP) ev.b |= B_DRAG;*/
+						xterm_button = -1;
+						if ((ev.b & BM_ACT) == B_DOWN) xterm_button = ev.b & BM_BUTT;
+						el += 3;
 					}
-					/*if ((itrm->kqueue[el] & 4) && ev.b != B_UP) ev.b |= B_DRAG;*/
-					xterm_button = -1;
-					if ((ev.b & BM_ACT) == B_DOWN) xterm_button = ev.b & BM_BUTT;
-					el += 3;
+					ev.ev = EV_MOUSE;
 					break;
 			}
 		} else {
@@ -534,6 +578,7 @@ int process_queue(struct itrm *itrm)
 				goto l2;
 			}
 			if (itrm->kqueue[1] == '\033') {
+				if (itrm->qlen >= 3 && (itrm->kqueue[2] == '[' || itrm->kqueue[2] == 'O')) el = 1;
 				ev.x = KBD_ESC;
 				goto l2;
 			}
@@ -550,11 +595,11 @@ int process_queue(struct itrm *itrm)
 	el = 1;
 	ev.x = itrm->kqueue[0];
 	l2:
-	if (ev.x == 1) ev.x = KBD_HOME;
+	/*if (ev.x == 1) ev.x = KBD_HOME;
 	if (ev.x == 2) ev.x = KBD_PAGE_UP;
 	if (ev.x == 4) ev.x = KBD_DEL;
 	if (ev.x == 5) ev.x = KBD_END;
-	if (ev.x == 6) ev.x = KBD_PAGE_DOWN;
+	if (ev.x == 6) ev.x = KBD_PAGE_DOWN;*/
 	if (ev.x == 8) ev.x = KBD_BS;
 	if (ev.x == 9) ev.x = KBD_TAB;
 	if (ev.x == 10) ev.x = KBD_ENTER, ev.y = KBD_CTRL;
