@@ -493,6 +493,7 @@ struct open_in_new {
 
 int get_system_env();
 int is_xterm();
+int can_twterm();
 int get_terminal_size(int, int *, int *);
 void handle_terminal_resize(int, void (*)());
 void unhandle_terminal_resize(int);
@@ -675,11 +676,12 @@ static inline int getpri(struct connection *c)
 #define S_WAIT		0
 #define S_DNS		1
 #define S_CONN		2
-#define S_SENT		3
-#define S_GETH		4
-#define S_PROC		5
-#define S_TRANS		6
-#define S_QUESTIONS	7
+#define S_SSL_NEG	3
+#define S_SENT		4
+#define S_GETH		5
+#define S_PROC		6
+#define S_TRANS		7
+#define S_QUESTIONS	8
 
 #define S_WAIT_REDIR		-999
 #define S_OK			-1000
@@ -947,6 +949,7 @@ struct window {
 #define ENV_SCREEN	2
 #define ENV_OS2VIO	4
 #define ENV_BE		8
+#define ENV_TWIN	16
 
 struct terminal {
 	struct terminal *next;
@@ -1703,6 +1706,7 @@ void free_conv_table();
 
 int can_open_in_new(struct terminal *);
 void open_in_new_window(struct terminal *, void (*)(struct terminal *, void (*)(struct terminal *, unsigned char *, unsigned char *), struct session *ses), struct session *);
+void send_open_in_new_xterm(struct terminal *term, void (*open_window)(struct terminal *term, unsigned char *, unsigned char *), struct session *ses);
 void send_open_new_xterm(struct terminal *, void (*)(struct terminal *, unsigned char *, unsigned char *), struct session *);
 void destroy_fc(struct form_control *);
 void sort_links(struct f_data *);
@@ -2086,7 +2090,8 @@ void launch_bm_add_doc_dialog(struct terminal *,struct dialog_data *,struct sess
 
 #define KM_MAIN		0
 #define KM_EDIT		1
-#define KM_MAX		2
+#define KM_MENU		2
+#define KM_MAX		3
 
 enum {
 	ACT_ADD_BOOKMARK,
@@ -2107,6 +2112,7 @@ enum {
 	ACT_FIND_NEXT_BACK,
 	ACT_GOTO_URL,
 	ACT_GOTO_URL_CURRENT,
+	ACT_GOTO_URL_CURRENT_LINK,
 	ACT_HEADER_INFO,
 	ACT_HOME,
 	ACT_KILL_TO_BOL,
@@ -2114,10 +2120,13 @@ enum {
 	ACT_LEFT,
 	ACT_MENU,
 	ACT_NEXT_FRAME,
+	ACT_OPEN_NEW_WINDOW,
+	ACT_OPEN_LINK_IN_NEW_WINDOW,
 	ACT_PAGE_DOWN,
 	ACT_PAGE_UP,
 	ACT_PASTE_CLIPBOARD,
 	ACT_PREVIOUS_FRAME,
+	ACT_REALLYQUIT,
 	ACT_QUIT,
 	ACT_RELOAD,
 	ACT_RIGHT,
@@ -2128,6 +2137,7 @@ enum {
 	ACT_SEARCH,
 	ACT_SEARCH_BACK,
 	ACT_TOGGLE_DISPLAY_IMAGES,
+	ACT_TOGGLE_DISPLAY_TABLES,
 	ACT_TOGGLE_HTML_PLAIN,
 	ACT_UP,
 	ACT_VIEW_IMAGE,
