@@ -67,7 +67,7 @@ void dont_exit_prog(struct session *ses)
 void query_exit(struct session *ses)
 {
 	ses->exit_query = 1;
-	msg_box(ses->term, NULL, TEXT(T_EXIT_LINKS), AL_LEFT, (ses->term->next == ses->term->prev && are_there_downloads()) ? TEXT(T_DO_YOU_REALLY_WANT_TO_EXIT_LINKS_AND_TERMINATE_ALL_DOWNLOADS) : TEXT(T_DO_YOU_REALLY_WANT_TO_EXIT_LINKS), ses, 2, TEXT(T_YES), (void (*)(void *))really_exit_prog, B_ENTER, TEXT(T_NO), dont_exit_prog, B_ESC);
+	msg_box(ses->term, NULL, TEXT(T_EXIT_LINKS), AL_CENTER, (ses->term->next == ses->term->prev && are_there_downloads()) ? TEXT(T_DO_YOU_REALLY_WANT_TO_EXIT_LINKS_AND_TERMINATE_ALL_DOWNLOADS) : TEXT(T_DO_YOU_REALLY_WANT_TO_EXIT_LINKS), ses, 2, TEXT(T_YES), (void (*)(void *))really_exit_prog, B_ENTER, TEXT(T_NO), dont_exit_prog, B_ESC);
 }
 
 void exit_prog(struct terminal *term, void *d, struct session *ses)
@@ -282,8 +282,11 @@ void history_menu(struct terminal *term, void *ddd, struct session *ses)
 	int n = 0;
 	foreach(l, ses->history) {
 		if (n/* || ses->tq_goto_position*/) {
+			unsigned char *url;
 			if (!mi && !(mi = new_menu(3))) return;
-			add_to_menu(&mi, stracpy(l->vs.url), "", "", MENU_FUNC go_backwards, (void *) n, 0);
+			url = stracpy(l->vs.url);
+			if (strchr(url, POST_CHAR)) *strchr(url, POST_CHAR) = 0;
+			add_to_menu(&mi, url, "", "", MENU_FUNC go_backwards, (void *) n, 0);
 		}
 		n++;
 	}
@@ -1202,3 +1205,120 @@ void free_history_lists()
 	free_list(search_history.items);
 }
 
+void auth_layout(struct dialog_data *dlg)
+{
+	struct terminal *term = dlg->win->term;
+	int max = 0, min = 0;
+	int w, rw;
+	int y = -1;
+	max_text_width(term, TEXT(T_USERID), &max);
+	min_text_width(term, TEXT(T_USERID), &min);
+	max_text_width(term, TEXT(T_PASSWORD), &max);
+	min_text_width(term, TEXT(T_PASSWORD), &min);
+	max_buttons_width(term, dlg->items + 2, 2,  &max);
+	min_buttons_width(term, dlg->items + 2, 2,  &min);
+	w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
+	if (w < min) w = min;
+	if (w > dlg->win->term->x - 2 * DIALOG_LB) w = dlg->win->term->x - 2 * DIALOG_LB;
+	if (w < 1) w = 1;
+	rw = 0;
+	if (dlg->dlg->udata) {
+		dlg_format_text(NULL, term, dlg->dlg->udata, 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
+		y++;
+	}
+
+	dlg_format_text(NULL, term, TEXT(T_USERID), 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
+	y += 2;
+	dlg_format_text(NULL, term, TEXT(T_PASSWORD), 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
+	y += 2;
+	dlg_format_buttons(NULL, term, dlg->items + 2, 2, 0, &y, w, &rw, AL_CENTER);
+	w = rw;
+	dlg->xw = w + 2 * DIALOG_LB;
+	dlg->yw = y + 2 * DIALOG_TB;
+	center_dlg(dlg);
+	draw_dlg(dlg);
+	y = dlg->y + DIALOG_TB;
+	if (dlg->dlg->udata) {
+		dlg_format_text(term, term, dlg->dlg->udata, dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
+		y++;
+	}
+	dlg_format_text(term, term, TEXT(T_USERID), dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_field(term, term, &dlg->items[0], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+	y++;
+	dlg_format_text(term, term, TEXT(T_PASSWORD), dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);	
+	dlg_format_field(term, term, &dlg->items[1], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+	y++;
+	dlg_format_buttons(term, term, &dlg->items[2], 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_CENTER);
+}
+
+int auth_ok(struct dialog_data *dlg, struct dialog_item_data *di)
+{
+	reload(dlg->dlg->refresh_data, -1);
+	return ok_dialog(dlg, di);
+}
+
+int auth_cancel(struct dialog_data *dlg, struct dialog_item_data *di)
+{
+	del_auth_entry(dlg->dlg->udata2);
+	return cancel_dialog(dlg, di);
+}
+
+extern struct list_head http_auth_basic_list;
+
+void do_auth_dialog(struct session *ses)
+{
+	struct dialog *d;
+	struct terminal *term = ses->term;
+	struct http_auth_basic *a = NULL;
+	if (!list_empty(http_auth_basic_list) && !((struct http_auth_basic*)http_auth_basic_list.next)->valid) a = (struct http_auth_basic*)http_auth_basic_list.next;
+	if (!a) return;
+	a->valid = 1;
+	if (!a->uid) {
+		if (!(a->uid = mem_alloc(MAX_UID_LEN))) {
+			del_auth_entry(a);
+			return;
+		}
+		*a->uid = 0;
+	}
+	if (!a->passwd) {
+		if (!(a->passwd = mem_alloc(MAX_PASSWD_LEN))) {
+			del_auth_entry(a);
+			return;
+		}
+		*a->passwd = 0;
+	}
+	if (!(d = mem_alloc(sizeof(struct dialog) + 5 * sizeof(struct dialog_item) + strlen(_(TEXT(T_ENTER_USERNAME), term))+(a->realm ? strlen(a->realm) : 0)+strlen(_(TEXT(T_AT), term))+strlen(a->url)+1))) return;
+	memset(d, 0, sizeof(struct dialog) + 5 * sizeof(struct dialog_item));
+	d->title = TEXT(T_PASSWORD);
+	d->fn = auth_layout;
+
+	d->udata = (char *)d + sizeof(struct dialog) + 5 * sizeof(struct dialog_item);
+	strcpy(d->udata, _(TEXT(T_ENTER_USERNAME), term));
+	if (a->realm) strcat(d->udata, a->realm);
+	strcat(d->udata, _(TEXT(T_AT), term));
+	strcat(d->udata, a->url);
+
+	d->udata2 = a;
+	d->refresh_data = ses;
+	
+	d->items[0].type = D_FIELD;
+	d->items[0].dlen = MAX_UID_LEN;
+	d->items[0].data = a->uid;
+	
+	d->items[1].type = D_FIELD_PASS;
+	d->items[1].dlen = MAX_PASSWD_LEN;
+	d->items[1].data = a->passwd;
+	
+	d->items[2].type = D_BUTTON;
+	d->items[2].gid = B_ENTER;
+	d->items[2].fn = auth_ok;
+	d->items[2].text = TEXT(T_OK);
+	
+	d->items[3].type = D_BUTTON;
+	d->items[3].gid = B_ESC;
+	d->items[3].fn = auth_cancel;
+	d->items[3].text = TEXT(T_CANCEL);
+
+	d->items[4].type = D_END;
+	do_dialog(term, d, getml(d, NULL));
+}
