@@ -513,15 +513,13 @@ void check_queue()
 #endif
 }
 
-int check_noproxy(unsigned char *url);
-
 unsigned char *get_proxy(unsigned char *url)
 {
 	int l = strlen(url);
 	unsigned char *proxy = NULL;
 	unsigned char *u;
-	if (*http_proxy && l >= 7 && !casecmp(url, "http://", 7) && !check_noproxy(url)) proxy = http_proxy;
-	if (*ftp_proxy && l >= 6 && !casecmp(url, "ftp://", 6) && !check_noproxy(url)) proxy = ftp_proxy;
+	if (*http_proxy && l >= 7 && !casecmp(url, "http://", 7)) proxy = http_proxy;
+	if (*ftp_proxy && l >= 6 && !casecmp(url, "ftp://", 6)) proxy = ftp_proxy;
 	if (!(u = mem_alloc(l + 1 + (proxy ? strlen(proxy) + 9 : 0)))) return NULL;
 	if (proxy) strcpy(u, "proxy://"), strcat(u, proxy), strcat(u, "/");
 	else *u = 0;
@@ -794,73 +792,6 @@ void free_blacklist()
 	free_list(blacklist);
 }
 
-struct noproxy {
-	unsigned char *st;
-	int len;
-} *noproxy_list = DUMMY;
-
-int check_noproxy(unsigned char *url)
-{
-	unsigned char *host;
-	struct noproxy *nl = noproxy_list;
-	if (nl == DUMMY) return 0;
-	if (!(host = get_host_name(url))) return 0;
-	while (nl->st) {
-		int l = strlen(host);
-		if ((l == nl->len) || (l > nl->len && host[l - nl->len - 1] == '.')) if (!strncasecmp(host + l - nl->len, nl->st, nl->len)) {
-			mem_free(host);
-			return 1;
-		}
-		nl++;
-	}
-	mem_free(host);
-	return 0;
-}
-
-void update_noproxy()
-{
-	int n = 0;
-	unsigned char *list = no_proxy_for;
-	struct noproxy *nl;
-	if (!*list) return;
-	while  (*list) {
-		if (*list == ',') n++;
-		list++;
-	}
-	if (!(nl = mem_realloc(noproxy_list, (n + 2) * sizeof(struct noproxy)))) return;
-	noproxy_list = nl;
-	list = no_proxy_for;
-	while(*list == ' '|| *list == ',') list++;
-	if (!*list) goto end;
-	nl->st = list;
-	while (*list) {
-		if (*list == ',') {
-			nl->len = list - nl->st;
-			while (*(nl->st + nl->len - 1) == ' ') {
-				if (!--nl->len) {
-					nl--;
-					break;
-				}
-			}
-			nl++;
-			list++;
-			while(*list == ' ' || *list == ',') list++;
-			if (!*list) goto end;
-			nl->st = list;
-		}
-		list++;
-	}
-	(nl++)->len = list - nl->st;
-	end:	
-	nl->st = 0;
-	nl->len  = 0;
-}
-
-void free_noproxy()
-{
-	mem_free(noproxy_list);
-}
-
 struct list_head http_auth_basic_list = { &http_auth_basic_list, &http_auth_basic_list };
 
 struct http_auth_basic *find_auth_entry(unsigned char *url)
@@ -966,6 +897,7 @@ struct s_msg_dsc msg_dsc[] = {
 	S_WAIT,			TEXT(T_WAITING_IN_QUEUE),
 	S_DNS,			TEXT(T_LOOKING_UP_HOST),
 	S_CONN,			TEXT(T_MAKING_CONNECTION),
+	S_SSL_NEG,		TEXT(T_SSL_NEGOTIATION),
 	S_SENT,			TEXT(T_REQUEST_SENT),
 	S_GETH,			TEXT(T_GETTING_HEADERS),
 	S_PROC,			TEXT(T_SERVER_IS_PROCESSING_REQUEST),
