@@ -64,8 +64,9 @@ void add_time_to_str(unsigned char **s, int *l, ttime t)
 	unsigned char q[64];
 	t /= 1000;
 	t &= 0xffffffff;
-	if (t >= 86400) sprintf(q, "%dd ", (int)(t / 86400)), add_to_str(s, l, q), t %= 86400;
-	if (t >= 3600) sprintf(q, "%d:%02d", (int)(t / 3600), (int)(t / 60 % 60)), add_to_str(s, l, q);
+	if (t < 0) t = 0;
+	if (t >= 86400) sprintf(q, "%dd ", (int)(t / 86400)), add_to_str(s, l, q);
+	if (t >= 3600) t %= 86400, sprintf(q, "%d:%02d", (int)(t / 3600), (int)(t / 60 % 60)), add_to_str(s, l, q);
 	else sprintf(q, "%d", (int)(t / 60)), add_to_str(s, l, q);
 	sprintf(q, ":%02d", (int)(t % 60)), add_to_str(s, l, q);
 }
@@ -1606,5 +1607,5 @@ unsigned char *get_current_link_url(struct session *ses, unsigned char *str, siz
     if (l->type != L_LINK) 
 		return NULL;
 	
-	return safe_strncpy(str, l->where, str_size);
+	return safe_strncpy(str, l->where ? l->where : l->where_img, str_size);
 }
