@@ -1,5 +1,9 @@
 #include "links.h"
 
+struct list_head mailto_prog = { &mailto_prog, &mailto_prog };
+struct list_head telnet_prog = { &telnet_prog, &telnet_prog };
+struct list_head tn3270_prog = { &tn3270_prog, &tn3270_prog };
+
 struct list_head assoc = { &assoc, &assoc };
 
 tcount get_assoc_cnt()
@@ -50,7 +54,7 @@ unsigned char *get_content_type(unsigned char *head, unsigned char *url)
 	struct assoc *a;
 	unsigned char *ct, *ext, *exxt;
 	int extl, el;
-	if (head && (ct = parse_http_header(head, "Content-Type"))) {
+	if (head && (ct = parse_http_header(head, "Content-Type", NULL))) {
 		unsigned char *s;
 		if ((s = strchr(ct, ';'))) *s = 0;
 		while (*ct && ct[strlen(ct) - 1] <= ' ') ct[strlen(ct) - 1] = 0;
@@ -75,7 +79,7 @@ unsigned char *get_content_type(unsigned char *head, unsigned char *url)
 struct assoc *get_type_assoc(struct terminal *term, unsigned char *type)
 {
 	struct assoc *a;
-	foreach(a, assoc) if (a->system == SYSTEM_ID && (term->xwin ? a->xwin : a->cons) && is_in_list(a->ct, type, strlen(type))) return a;
+	foreach(a, assoc) if (a->system == SYSTEM_ID && (term->environment & ENV_XWIN ? a->xwin : a->cons) && is_in_list(a->ct, type, strlen(type))) return a;
 	return NULL;
 }
 
@@ -83,6 +87,7 @@ void free_types()
 {
 	struct assoc *a;
 	struct extension *e;
+	struct protocol_program *p;
 	foreach(a, assoc) {
 		mem_free(a->ct);
 		mem_free(a->prog);
@@ -94,65 +99,84 @@ void free_types()
 		mem_free(e->ct);
 	}
 	free_list(extensions);
+	foreach(p, mailto_prog) mem_free(p->prog);
+	free_list(mailto_prog);
+	foreach(p, telnet_prog) mem_free(p->prog);
+	free_list(telnet_prog);
+	foreach(p, tn3270_prog) mem_free(p->prog);
+	free_list(tn3270_prog);
 }
 
 unsigned char *ct_msg[] = {
-	"Label",
-	"Content-Type(s)",
-	"Program ('%' is replaced with file name)",
-	"Run on terminal",
-	"Run in X-Window",
-	"Ask before opening",
+	TEXT(T_LABEL),
+	TEXT(T_CONTENT_TYPES),
+	TEXT(T_PROGRAM__IS_REPLACED_WITH_FILE_NAME),
+#ifdef ASSOC_BLOCK
+	TEXT(T_BLOCK_TERMINAL_WHILE_PROGRAM_RUNNING),
+#endif
+#ifdef ASSOC_CONS_XWIN
+	TEXT(T_RUN_ON_TERMINAL),
+	TEXT(T_RUN_IN_XWINDOW),
+#endif
+	TEXT(T_ASK_BEFORE_OPENING),
 };
 
 void add_ct_fn(struct dialog_data *dlg)
 {
+	struct terminal *term = dlg->win->term;
 	int max = 0, min = 0;
 	int w, rw;
-	int x, y = -1;
-	max_text_width(ct_msg[0], &max);
-	min_text_width(ct_msg[0], &min);
-	max_text_width(ct_msg[1], &max);
-	min_text_width(ct_msg[1], &min);
-	max_text_width(ct_msg[2], &max);
-	min_text_width(ct_msg[2], &min);
-	max_group_width(ct_msg + 3, dlg->items + 3, 3, &max);
-	min_group_width(ct_msg + 3, dlg->items + 3, 3, &min);
-	max_buttons_width(dlg->items + 6, 2, &max);
-	min_buttons_width(dlg->items + 6, 2, &min);
-	w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
+	int y = -1;
+	int p = 1;
+#ifdef ASSOC_BLOCK
+	p++;
+#endif
+#ifdef ASSOC_CONS_XWIN
+	p += 2;
+#endif
+	max_text_width(term, ct_msg[0], &max);
+	min_text_width(term, ct_msg[0], &min);
+	max_text_width(term, ct_msg[1], &max);
+	min_text_width(term, ct_msg[1], &min);
+	max_text_width(term, ct_msg[2], &max);
+	min_text_width(term, ct_msg[2], &min);
+	max_group_width(term, ct_msg + 3, dlg->items + 3, p, &max);
+	min_group_width(term, ct_msg + 3, dlg->items + 3, p, &min);
+	max_buttons_width(term, dlg->items + 3 + p, 2, &max);
+	min_buttons_width(term, dlg->items + 3 + p, 2, &min);
+	w = term->x * 9 / 10 - 2 * DIALOG_LB;
 	if (w > max) w = max;
 	if (w < min) w = min;
-	if (w > dlg->win->term->x - 2 * DIALOG_LB) w = dlg->win->term->x - 2 * DIALOG_LB;
+	if (w > term->x - 2 * DIALOG_LB) w = term->x - 2 * DIALOG_LB;
 	if (w < 1) w = 1;
 	rw = 0;
-	dlg_format_text(NULL, ct_msg[0], 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_text(NULL, term, _(ct_msg[0], term), 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
 	y += 2;
-	dlg_format_text(NULL, ct_msg[1], 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_text(NULL, term, _(ct_msg[1], term), 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
 	y += 2;
-	dlg_format_text(NULL, ct_msg[2], 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_text(NULL, term, _(ct_msg[2], term), 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
 	y += 2;
-	dlg_format_group(NULL, ct_msg + 3, dlg->items + 3, 3, 0, &y, w, &rw);
+	dlg_format_group(NULL, term, ct_msg + 3, dlg->items + 3, p, 0, &y, w, &rw);
 	y++;
-	dlg_format_buttons(NULL, dlg->items + 6, 2, 0, &y, w, &rw, AL_CENTER);
+	dlg_format_buttons(NULL, term, dlg->items + 3 + p, 2, 0, &y, w, &rw, AL_CENTER);
 	w = rw;
 	dlg->xw = w + 2 * DIALOG_LB;
 	dlg->yw = y + 2 * DIALOG_TB;
 	center_dlg(dlg);
 	draw_dlg(dlg);
 	y = dlg->y + DIALOG_TB;
-	dlg_format_text(dlg->win->term, ct_msg[0], dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
-	dlg_format_field(dlg->win->term, &dlg->items[0], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+	dlg_format_text(term, term, ct_msg[0], dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_field(term, term, &dlg->items[0], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
 	y++;
-	dlg_format_text(dlg->win->term, ct_msg[1], dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
-	dlg_format_field(dlg->win->term, &dlg->items[1], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+	dlg_format_text(term, term, ct_msg[1], dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_field(term, term, &dlg->items[1], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
 	y++;
-	dlg_format_text(dlg->win->term, ct_msg[2], dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
-	dlg_format_field(dlg->win->term, &dlg->items[2], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+	dlg_format_text(term, term, ct_msg[2], dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_field(term, term, &dlg->items[2], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
 	y++;
-	dlg_format_group(dlg->win->term, ct_msg + 3, &dlg->items[3], 3, dlg->x + DIALOG_LB, &y, w, NULL);
+	dlg_format_group(term, term, ct_msg + 3, &dlg->items[3], p, dlg->x + DIALOG_LB, &y, w, NULL);
 	y++;
-	dlg_format_buttons(dlg->win->term, &dlg->items[6], 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_CENTER);
+	dlg_format_buttons(term, term, &dlg->items[3 + p], 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_CENTER);
 }
 
 void update_assoc(struct assoc *new)
@@ -175,6 +199,7 @@ void update_assoc(struct assoc *new)
 	repl->label = stracpy(new->label);
 	repl->ct = stracpy(new->ct);
 	repl->prog = stracpy(new->prog);
+	repl->block = new->block;
 	repl->cons = new->cons;
 	repl->xwin = new->xwin;
 	repl->ask = new->ask;
@@ -202,18 +227,15 @@ void menu_del_ct(struct terminal *term, void *fcp, void *xxx2)
 	return;
 	ok:
 	str = init_str(), l = 0;
-	add_to_str(&str, &l, get_text("Delete association "));
-	add_to_str(&str, &l, del->label);
-	add_to_str(&str, &l, ": ");
 	add_to_str(&str, &l, del->ct);
 	add_to_str(&str, &l, " -> ");
 	add_to_str(&str, &l, del->prog);
-	add_to_str(&str, &l, "?");
-	msg_box(term, getml(str, NULL), get_text("Delete association"), AL_CENTER, str, fcp, 2, get_text("Yes"), really_del_ct, B_ENTER, get_text("No"), NULL, B_ESC);
+	msg_box(term, getml(str, NULL), TEXT(T_DELETE_ASSOCIATION), AL_CENTER | AL_EXTD_TEXT, TEXT(T_DELETE_ASSOCIATION), ": ", str, "?", NULL, fcp, 2, TEXT(T_YES), really_del_ct, B_ENTER, TEXT(T_NO), NULL, B_ESC);
 }
 
 void menu_add_ct(struct terminal *term, void *fcp, void *xxx2)
 {
+	int p;
 	int fc = (int)fcp;
 	struct assoc *new, *from;
 	unsigned char *label;
@@ -226,9 +248,9 @@ void menu_add_ct(struct terminal *term, void *fcp, void *xxx2)
 	}
 	from = NULL;
 	ok:
-	if (!(d = mem_alloc(sizeof(struct dialog) + 9 * sizeof(struct dialog_item) + sizeof(struct assoc) + 3 * MAX_STR_LEN))) return;
-	memset(d, 0, sizeof(struct dialog) + 9 * sizeof(struct dialog_item) + sizeof(struct assoc) + 3 * MAX_STR_LEN);
-	new = (struct assoc *)&d->items[9];
+	if (!(d = mem_alloc(sizeof(struct dialog) + 10 * sizeof(struct dialog_item) + sizeof(struct assoc) + 3 * MAX_STR_LEN))) return;
+	memset(d, 0, sizeof(struct dialog) + 10 * sizeof(struct dialog_item) + sizeof(struct assoc) + 3 * MAX_STR_LEN);
+	new = (struct assoc *)&d->items[10];
 	new->label = label = (unsigned char *)(new + 1);
 	new->ct = ct = label + MAX_STR_LEN;
 	new->prog = prog = ct + MAX_STR_LEN;
@@ -236,17 +258,18 @@ void menu_add_ct(struct terminal *term, void *fcp, void *xxx2)
 		strncpy(label, from->label, MAX_STR_LEN - 1);
 		strncpy(ct, from->ct, MAX_STR_LEN - 1);
 		strncpy(prog, from->prog, MAX_STR_LEN - 1);
+		new->block = from->block;
 		new->cons = from->cons;
 		new->xwin = from->xwin;
 		new->ask = from->ask;
 		new->system = from->system;
 		new->cnt = from->cnt;
 	} else {
-		new->xwin = new->cons = 1;
+		new->block = new->xwin = new->cons = 1;
 		new->ask = 1;
 		new->system = SYSTEM_ID;
 	}
-	d->title = get_text("Association");
+	d->title = TEXT(T_ASSOCIATION);
 	d->fn = add_ct_fn;
 	d->refresh = (void (*)(void *))update_assoc;
 	d->refresh_data = new;
@@ -262,78 +285,94 @@ void menu_add_ct(struct terminal *term, void *fcp, void *xxx2)
 	d->items[2].dlen = MAX_STR_LEN;
 	d->items[2].data = prog;
 	d->items[2].fn = check_nonempty;
-	d->items[3].type = D_CHECKBOX;
-	d->items[3].data = (unsigned char *)&new->cons;
-	d->items[3].dlen = sizeof(int);
-	d->items[4].type = D_CHECKBOX;
-	d->items[4].data = (unsigned char *)&new->xwin;
-	d->items[4].dlen = sizeof(int);
-	d->items[5].type = D_CHECKBOX;
-	d->items[5].data = (unsigned char *)&new->ask;
-	d->items[5].dlen = sizeof(int);
-	d->items[6].type = D_BUTTON;
-	d->items[6].gid = B_ENTER;
-	d->items[6].fn = ok_dialog;
-	d->items[6].data = get_text("OK");
-	d->items[7].type = D_BUTTON;
-	d->items[7].gid = B_ESC;
-	d->items[7].data = get_text("Cancel");
-	d->items[7].fn = cancel_dialog;
-	d->items[8].type = D_END;
+	p = 3;
+#ifdef ASSOC_BLOCK
+	d->items[p].type = D_CHECKBOX;
+	d->items[p].data = (unsigned char *)&new->block;
+	d->items[p++].dlen = sizeof(int);
+#endif
+#ifdef ASSOC_CONS_XWIN
+	d->items[p].type = D_CHECKBOX;
+	d->items[p].data = (unsigned char *)&new->cons;
+	d->items[p++].dlen = sizeof(int);
+	d->items[p].type = D_CHECKBOX;
+	d->items[p].data = (unsigned char *)&new->xwin;
+	d->items[p++].dlen = sizeof(int);
+#endif
+	d->items[p].type = D_CHECKBOX;
+	d->items[p].data = (unsigned char *)&new->ask;
+	d->items[p++].dlen = sizeof(int);
+	d->items[p].type = D_BUTTON;
+	d->items[p].gid = B_ENTER;
+	d->items[p].fn = ok_dialog;
+	d->items[p++].text = TEXT(T_OK);
+	d->items[p].type = D_BUTTON;
+	d->items[p].gid = B_ESC;
+	d->items[p].text = TEXT(T_CANCEL);
+	d->items[p++].fn = cancel_dialog;
+	d->items[p++].type = D_END;
 	do_dialog(term, d, getml(d, NULL));
 }
 
+struct menu_item mi_no_assoc[] = {
+	TEXT(T_NO_ASSOCIATIONS), "", M_BAR, NULL, NULL, 0, 0,
+	NULL, NULL, 0, NULL, NULL, 0, 0
+};
+
 void menu_list_assoc(struct terminal *term, void *fn, void *xxx)
 {
 	struct assoc *a;
-	struct menu_item *mi;
+	struct menu_item *mi = NULL;
 	int n = 0;
-	if (!(mi = new_menu(7))) return;
-	foreachback(a, assoc) if (a->system == SYSTEM_ID) add_to_menu(&mi, stracpy(a->label), stracpy(a->ct), 0, MENU_FUNC fn, (void *)a->cnt, 0), n++;
-	if (!n) add_to_menu(&mi, stracpy(get_text("No associations")), stracpy(""), M_BAR, NULL, NULL, 0);
-	do_menu(term, mi, xxx);
+	foreachback(a, assoc) if (a->system == SYSTEM_ID) {
+		if (!mi && !(mi = new_menu(7))) return;
+		add_to_menu(&mi, stracpy(a->label), stracpy(a->ct), "", MENU_FUNC fn, (void *)a->cnt, 0), n++;
+	}
+	if (!mi) do_menu(term, mi_no_assoc, xxx);
+	else do_menu(term, mi, xxx);
 }
 
 unsigned char *ext_msg[] = {
-	"Extension(s)",
-	"Content-Type",
+	TEXT(T_EXTENSION_S),
+	TEXT(T_CONTENT_TYPE),
 };
 
 void add_ext_fn(struct dialog_data *dlg)
 {
+	struct terminal *term = dlg->win->term;
 	int max = 0, min = 0;
 	int w, rw;
-	int x, y = -1;
-	max_text_width(ext_msg[0], &max);
-	min_text_width(ext_msg[0], &min);
-	max_text_width(ext_msg[1], &max);
-	min_text_width(ext_msg[1], &min);
-	max_buttons_width(dlg->items + 2, 2, &max);
-	min_buttons_width(dlg->items + 2, 2, &min);
-	w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
+	int y = -1;
+	max_text_width(term, ext_msg[0], &max);
+	min_text_width(term, ext_msg[0], &min);
+	max_text_width(term, ext_msg[1], &max);
+	min_text_width(term, ext_msg[1], &min);
+	max_buttons_width(term, dlg->items + 2, 2, &max);
+	min_buttons_width(term, dlg->items + 2, 2, &min);
+	w = term->x * 9 / 10 - 2 * DIALOG_LB;
 	if (w > max) w = max;
 	if (w < min) w = min;
-	if (w > dlg->win->term->x - 2 * DIALOG_LB) w = dlg->win->term->x - 2 * DIALOG_LB;
+	if (w > term->x - 2 * DIALOG_LB) w = term->x - 2 * DIALOG_LB;
 	if (w < 1) w = 1;
 	rw = 0;
-	dlg_format_text(NULL, ext_msg[0], 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_text(NULL, term, ext_msg[0], 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
 	y += 2;
-	dlg_format_text(NULL, ext_msg[1], 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_text(NULL, term, ext_msg[1], 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
 	y += 2;
-	dlg_format_buttons(NULL, dlg->items + 2, 2, 0, &y, w, &rw, AL_CENTER);
+	dlg_format_buttons(NULL, term, dlg->items + 2, 2, 0, &y, w, &rw, AL_CENTER);
 	w = rw;
 	dlg->xw = w + 2 * DIALOG_LB;
 	dlg->yw = y + 2 * DIALOG_TB;
 	center_dlg(dlg);
 	draw_dlg(dlg);
 	y = dlg->y + DIALOG_TB;
-	dlg_format_text(dlg->win->term, ext_msg[0], dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
-	dlg_format_field(dlg->win->term, &dlg->items[0], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+	dlg_format_text(term, term, ext_msg[0], dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_field(term, term, &dlg->items[0], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
 	y++;
-	dlg_format_text(dlg->win->term, ext_msg[1], dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
-	dlg_format_field(dlg->win->term, &dlg->items[1], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+	dlg_format_text(term, term, ext_msg[1], dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_field(term, term, &dlg->items[1], dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
 	y++;
-	dlg_format_buttons(dlg->win->term, &dlg->items[2], 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_CENTER);
+	dlg_format_buttons(term, term, &dlg->items[2], 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_CENTER);
 }
 
 void update_ext(struct extension *new)
@@ -377,12 +416,10 @@ void menu_del_ext(struct terminal *term, void *fcp, void *xxx2)
 	return;
 	ok:
 	str = init_str(), l = 0;
-	add_to_str(&str, &l, get_text("Delete extension "));
 	add_to_str(&str, &l, del->ext);
 	add_to_str(&str, &l, " -> ");
 	add_to_str(&str, &l, del->ct);
-	add_to_str(&str, &l, "?");
-	msg_box(term, getml(str, NULL), get_text("Delete extension"), AL_CENTER, str, fcp, 2, get_text("Yes"), really_del_ext, B_ENTER, get_text("No"), NULL, B_ESC);
+	msg_box(term, getml(str, NULL), TEXT(T_DELETE_EXTENSION), AL_CENTER | AL_EXTD_TEXT, TEXT(T_DELETE_EXTENSION), " ", str, "?", NULL, fcp, 2, TEXT(T_YES), really_del_ext, B_ENTER, TEXT(T_NO), NULL, B_ESC);
 }
 
 void menu_add_ext(struct terminal *term, void *fcp, void *xxx2)
@@ -408,7 +445,7 @@ void menu_add_ext(struct terminal *term, void *fcp, void *xxx2)
 		strncpy(ct, from->ct, MAX_STR_LEN - 1);
 		new->cnt = from->cnt;
 	}
-	d->title = get_text("Extension");
+	d->title = TEXT(T_EXTENSION);
 	d->fn = add_ext_fn;
 	d->refresh = (void (*)(void *))update_ext;
 	d->refresh_data = new;
@@ -423,23 +460,55 @@ void menu_add_ext(struct terminal *term, void *fcp, void *xxx2)
 	d->items[2].type = D_BUTTON;
 	d->items[2].gid = B_ENTER;
 	d->items[2].fn = ok_dialog;
-	d->items[2].data = get_text("OK");
+	d->items[2].text = TEXT(T_OK);
 	d->items[3].type = D_BUTTON;
 	d->items[3].gid = B_ESC;
-	d->items[3].data = get_text("Cancel");
+	d->items[3].text = TEXT(T_CANCEL);
 	d->items[3].fn = cancel_dialog;
 	d->items[4].type = D_END;
 	do_dialog(term, d, getml(d, NULL));
 }
 
+struct menu_item mi_no_ext[] = {
+	TEXT(T_NO_EXTENSIONS), "", M_BAR, NULL, NULL, 0, 0,
+	NULL, NULL, 0, NULL, NULL, 0, 0
+};
+
 void menu_list_ext(struct terminal *term, void *fn, void *xxx)
 {
 	struct extension *a;
-	struct menu_item *mi;
+	struct menu_item *mi = NULL;
 	int n = 0;
-	if (!(mi = new_menu(7))) return;
-	foreachback(a, extensions) add_to_menu(&mi, stracpy(a->ext), stracpy(a->ct), 0, MENU_FUNC fn, (void *)a->cnt, 0), n++;
-	if (!n) add_to_menu(&mi, stracpy(get_text("No extensions")), stracpy(""), M_BAR, NULL, NULL, 0);
-	do_menu(term, mi, xxx);
+	foreachback(a, extensions) {
+		if (!mi && !(mi = new_menu(7))) return;
+		add_to_menu(&mi, stracpy(a->ext), stracpy(a->ct), "", MENU_FUNC fn, (void *)a->cnt, 0), n++;
+	}
+	if (!mi) do_menu(term, mi_no_ext, xxx);
+	else do_menu(term, mi, xxx);
 }
 
+void update_prog(struct list_head *l, unsigned char *p, int s)
+{
+	struct protocol_program *repl;
+	foreach(repl, *l) if (repl->system == s) {
+		mem_free(repl->prog);
+		goto ss;
+	}
+	if (!(repl = mem_alloc(sizeof(struct protocol_program)))) return;
+	add_to_list(*l, repl);
+	repl->system = s;
+	ss:
+	if ((repl->prog = mem_alloc(MAX_STR_LEN))) {
+		strncpy(repl->prog, p, MAX_STR_LEN);
+		repl->prog[MAX_STR_LEN - 1] = 0;
+	}
+}
+
+unsigned char *get_prog(struct list_head *l)
+{
+	struct protocol_program *repl;
+	foreach(repl, *l) if (repl->system == SYSTEM_ID) return repl->prog;
+	update_prog(l, "", SYSTEM_ID);
+	foreach(repl, *l) if (repl->system == SYSTEM_ID) return repl->prog;
+	return NULL;
+}
